#!/usr/bin/perl

###############################################################################
#
# Initialisierung
#
push (@INC, "./");
require ("config.pl");
require ("channels.pl");

push (@INC, "./inc");
require ("helperfunc.pl");

use LWP;
use Date::Manip;
use MD5;

&Date_Init("Language=German","DateFormat=non-US");

use Getopt::Std;
use IO::Socket;
use HTML::Entities;

my($SOCKET,$SVDRP); 
our $please_exit = 0;
$useragent = LWP::UserAgent->new;

$today = UnixDate("heute", "%s");
$next_month = UnixDate(DateCalc("heute","+ 1 Monat - 1 Tag"), "%s");
$timer_year = UnixDate("heute", "%Y");
$MarginStart *= 60;
$MarginStop *= 60;

@list = Date_Init();

$SIG{'INT'}  = \&SIGhandler;

###############################################################################
#
# Main
#
$Usage = qq{
Usage: $0 [options]

Options: -d hostname           destination hostname (default: localhost)
         -p port               SVDRP port number (default: 2001)
         -h	                   Show this help text
         -v                    Show verbose messages
         -s	                   Smulation Mode
};

$sim = $get = $verbose = 0;
die $Usage if (!getopts('d:p:hvs') || $opt_h);

$verbose = 1 if $opt_v;
$sim = 1 if $opt_s;
$Dest = $opt_d  || "localhost";
$Port = $opt_p  || 2001;

if ($cleanupoldfiles) {
	print "cleanup old files ..\n";
	cleanup();
}

print "reading channels ..\n";
our $SVDRP = SVDRP->new($Dest,$Port,$verbose,$sim);
my @channels = getchan();
doexit() if $please_exit;

$SVDRP->close;

# Merkzettel holen
print "getting timers ...\n";

$md5password = MD5->hexhash($password);
use LWP::UserAgent;
my $ua = new LWP::UserAgent; $ua->agent("Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) Opera 7.60  [de] (IBM EVV/3.0/EAK01AG9/LE)");
$ua->proxy('http', $http_proxy);
my $req = new HTTP::Request(GET => "$merkzettelURL?username=$username&password=$md5password");
my $res = $ua->request($req);
$success = 1 if $res->is_success;
doexit() if $please_exit;
$merkzettelgz = $res->content;

# Ich wei nicht, ob die Datei komprimiert ist oder nicht - egal.
open OUT, "> $tmp/merkzettel.xml.gz" || die "cannot write $tmp/merkzettel.gz: $!";
print OUT $merkzettelgz;
close OUT;
system("gunzip", "-f", "$tmp/merkzettel.xml.gz");
rename "$tmp/merkzettel.xml.gz","$tmp/merkzettel.xml" if (-e "$tmp/merkzettel.xml.gz");

die "empty file - wrong username/password?" if (! -s "$tmp/merkzettel.xml");
print "got merkzettel\n";

print "parsing timers ...\n";
($id,$channel,$start,$ende,$title,$nature,$format,$director,$actor,$fulltitle)="";
open IN, "<$tmp/merkzettel.xml" || die "cannot read $tmp/merkzettel.xml: $!";
while (<IN>) {
	doexit() if $please_exit;
	chomp;
	($id,$channel,$start,$end) = ($1,$2,$3,$4) if (/<epg_schedule_entry uid="(.*?)" channel="(.*?)" starttime="(.*?)" endtime="(.*?)" eventtype="rec">/);
	$title = $1 if (/<title><!\[CDATA\[(.*?)\]\]><\/title>/);
	$nature = $1 if (/<nature>(.*?)<\/nature>/);
	$format = $1 if (/<format>(.*?)<\/format>/);
	$director = $1 if (/<cast_director>(.*?)<\/cast_director>/);
	$actor = $1 if (/<cast_actors>(.*?)<\/cast_actors>/);

	# Ende eines Eintrags gefunden?
	if (/<\/epg_schedule_entry>/) {
		foreach $ts (keys(%chan)) {
			if ($chan{$ts}[4] eq $channel) {
				$epg_chan_id = $chan{$ts}[0];
				$rfc2838 = $ts;
				last;
			}
		}
		$timerstart = UnixDate(ParseDate($start), "%s") - $MarginStart;
		$timerstop  = UnixDate(ParseDate($end),   "%s") + $MarginStop;

		$anfang = &ParseDateString("epoch $timerstart");
		$ende = &ParseDateString("epoch $timerstop");
		$timer_day = &UnixDate($anfang, "%d");
		$anfang = UnixDate($anfang, "%H%M");
		$ende = UnixDate($ende, "%H%M");
		$anfang =~ s/://;
		$ende =~ s/://;

		next if ($timerstop < $today);
		next if ($timerstart > $next_month);

		$format =~s/\s*(.*?)\s*$/$1/g;
		$fulltitle = $format . "~" if ($format ne "");
		$nature =~s/\s*(.*?)\s*$/$1/g;
		$fulltitle .= $nature . "~" if ($nature ne "");
		$title =~s/\s*(.*?)\s*$/$1/g;
		$fulltitle = $tvinfoprefix . $fulltitle . $title;
		
		$channel_id = "";
		foreach $channel (@channels) {
			$data = $channel->{nid}>0 ? $channel->{tid} : $channel->{frequency};
			if ("$channel->{source}-$channel->{nid}-$data-$channel->{service_id}" eq "$epg_chan_id") {
				$channel_id = $channel->{vdr_id};
				}  
			}
			push(@newtimers, {
				channel_id => $channel_id,
				timer_day  => $timer_day,
				anfang     => $anfang,
				ende       => $ende,
				genre      => "$format $nature",
				title      => $fulltitle,
				summary    => "",
				subtitle   => ""
			}); 
		
		($id,$channel,$start,$ende,$title,$nature,$format,$director,$actor,$fulltitle)="";
		}
	}

# unlink "$tmp/merkzettel.gz";

# alte tvinfo-timer lschen (Timer, die gerade aufzeichnen, lscht vdr nicht)
our $SVDRP = SVDRP->new($Dest,$Port,$verbose,$sim);
print "reading old timers\n";
my @oldtimers = gettimers($tvinfoprefix);

print "deleting ".($#oldtimers+1)." old Timers\n";
deltimers((@oldtimers));

# und neue Timer eintragen
foreach $timerevent (@newtimers) {
	print "programming $timerevent->{title}\n";
	doexit() if $please_exit;
	my($result) = $SVDRP->SendCMD("newt 1:$timerevent->{channel_id}:$timerevent->{timer_day}:$timerevent->{anfang}:$timerevent->{ende}:$prio:$lifetime:$timerevent->{title}:$timerevent->{summary}");
	if ($result =~ m/^(\d+)\s+1:/) {
		print "programmed timer $1\n";
	} else {
		print "Error programming new timer:$result\n";
	}
}
$SVDRP->close;
exit;

